package gov.llnl.babel.backend.jdk;

import gov.llnl.babel.BabelConfiguration;
import gov.llnl.babel.backend.CodeConstants;
import gov.llnl.babel.backend.CodeGenerationException;
import gov.llnl.babel.backend.CodeGenerator;
import gov.llnl.babel.backend.CodeSplicer;
import gov.llnl.babel.backend.jdk.Java;
import gov.llnl.babel.backend.FileManager;
import gov.llnl.babel.backend.IOR;
import gov.llnl.babel.backend.Utilities;
import gov.llnl.babel.backend.writers.LanguageWriter;
import gov.llnl.babel.backend.writers.LanguageWriterForJava;
import gov.llnl.babel.symbols.Argument;
import gov.llnl.babel.symbols.Class;
import gov.llnl.babel.symbols.Comment;
import gov.llnl.babel.symbols.Extendable;
import gov.llnl.babel.symbols.Method;
import gov.llnl.babel.symbols.Symbol;
import gov.llnl.babel.symbols.SymbolID;
import gov.llnl.babel.symbols.SymbolUtilities;
import gov.llnl.babel.symbols.Type;
import gov.llnl.babel.symbols.Version;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;


public class JavaImplSource {
  private Extendable d_ext = null;
  private LanguageWriterForJava d_writer = null;
  private CodeSplicer d_splicer = null;
  
  public JavaImplSource(Extendable ext, LanguageWriterForJava lw) {
    d_ext = ext;
    d_writer = lw;
  
//     SymbolID id = symbol.getSymbolID();
//     String filename = getJavaImplSourceFile(symbol);
//     LanguageWriterForJava d_writer = new LanguageWriterForJava(FileManager.getInstance().
//                                                          createFile(symbol,filegroup,filename));
//     d_writer.writeBanner(symbol, filename, (role == FILE_ROLE_IMPL), generateRoleDescription(id, role) );
//   }

   
  }
  
  public static void generateCode(Extendable ext, LanguageWriterForJava writer) 
    throws CodeGenerationException
  {
    JavaImplSource source = new JavaImplSource(ext, writer);
    source.generateCode(ext);
  }
  

  /**
   * Write Java impl information for the provided symbol to the language
   * writer output stream provided in the class constructor.  This method
   * does not close the writer output stream. 
   */
  public void generateCode(Extendable symbol) 
    throws CodeGenerationException 
  {
    final SymbolID id = d_ext.getSymbolID();
    final int type = d_ext.getSymbolType();
    String filename = Java.getJavaImplSourceFile( id );
    //System.out.println("Create " + filename + "..." );
    
    try {
      d_splicer = FileManager.getInstance().getCodeSplicer(id, type, filename);
      //d_writer = Java.createSource( d_ext ); // creates language writer
        
      writePackage();
        
      writeImports();
        
      spliceImports();
        
      writeClassBeginning();
        
      writeCtorDtor();
        
      writeSIDLDefinedMethods();
        
      d_splicer.splice( d_ext.getSymbolID().getFullName() + "._misc",
                        d_writer, "Put miscellaneous code here");
      d_writer.println();
        
      writeClassEnd();
        
      if (d_splicer.hasUnusedSymbolEdits()) {
        d_writer.beginBlockComment(true);
        d_writer.println(CodeConstants.C_BEGIN_UNREFERENCED_METHODS);
        d_writer.println(CodeConstants.C_UNREFERENCED_COMMENT1);
        d_writer.println(CodeConstants.C_UNREFERENCED_COMMENT2);
        d_writer.println(CodeConstants.C_UNREFERENCED_COMMENT3);
        d_writer.endBlockComment(true);
        d_splicer.outputUnusedSymbolEdits( d_writer.getPrintWriter() );
        d_writer.writeCommentLine(CodeConstants.C_END_UNREFERENCED_METHODS);
      }
    } catch ( java.io.IOException ex) { 
      throw new CodeGenerationException("IOException : " + ex.getMessage() );
    } finally {
      if (d_writer != null) {
        d_writer.close();
        d_writer = null;
      }
    }
  }
  
  private void writePackage() {
    String pkg = SymbolUtilities.getParentPackage(d_ext.getSymbolID().getFullName());
    d_writer.println("package " + pkg + ";");
    d_writer.println();
  }
  
  private void writeImports() {
    Iterator i = d_ext.getSymbolReferences().iterator();
    while (i.hasNext()) {
	    SymbolID id = (SymbolID) i.next();
      if (! id.getFullName().equals(d_ext.getSymbolID().getFullName())) {
        d_writer.println("import " + id.getFullName() + ";");
      }
    }
    d_writer.println();
  }
  
  private void spliceImports() {
    d_splicer.splice(d_ext.getSymbolID().getFullName() + ".imports",
                     d_writer, "Put additional imports here...");
    d_writer.println();
  }

  private void writeClassBeginning() {
    String name = d_ext.getSymbolID().getShortName();
    SymbolID id = d_ext.getSymbolID();
    String splicer_symbol_impl = d_ext.getSymbolID().getFullName() + "._implementation";
    String splicer_symbol_inherit = d_ext.getSymbolID().getFullName() + "._inherits";
    d_writer.writeComment(d_ext, true);
    d_writer.println("public class " + name);
    d_splicer.splice(splicer_symbol_inherit, d_writer, 
                     "Put additional inheritance here...");
    d_writer.println("{");
    d_writer.println();
    d_writer.increaseTabLevel();
    d_writer.writeCommentLine("Pointer back to IOR.");
    d_writer.writeCommentLine("Use this to dispatch back through IOR vtable");
    d_writer.println("private " + name + " IORself;");
    d_writer.println();
	
    d_splicer.splice(splicer_symbol_impl, d_writer, 
                     "Put aditional implementation details here...");
    d_writer.println();
	
    d_writer.writeCommentLine("private default constructor (required)");
    d_writer.println("private " + name + "() {} ");
    d_writer.println();

//     d_writer.writeCommentLine("SIDL constructor (required)");
//     d_writer.println(name + "( " + IOR.getObjectName(id) +
//                      " s) { s = true; _ctor(); }");
//     d_writer.println();

  }

  private void writeClassEnd() {
    String name = d_ext.getSymbolID().getShortName();
    d_writer.decreaseTabLevel();
    d_writer.print("}; "); // end of class
    d_writer.writeCommentLine("end class " + name );
    d_writer.println();
  }

  private void writeCtorDtor() {
    d_writer.writeCommentLine("user defined constructor");
    d_writer.println("public void _ctor() {");
    d_writer.increaseTabLevel();
    d_splicer.splice(d_ext.getSymbolID().getFullName() + "._ctor",
                     d_writer,
                     "add construction details here");
    d_writer.decreaseTabLevel();
    d_writer.println("}");
    d_writer.println();

    d_writer.writeCommentLine("user defined destructor");
    d_writer.println("public void _dtor() {");
    d_writer.increaseTabLevel();
    d_splicer.splice(d_ext.getSymbolID().getFullName() + "._dtor",
                     d_writer,
                     "add destruction details here");
    d_writer.decreaseTabLevel();
    d_writer.println("}");
    d_writer.println();

  }

  private void writeSIDLDefinedMethods() {
    List static_methods = (List) d_ext.getStaticMethods(false);
    if (static_methods.size() > 0) {
	    d_writer.writeCommentLine("user defined static methods:");
	    for (Iterator m = static_methods.iterator(); m.hasNext(); ) {
        Method method = (Method) m.next();
        generateMethod(method);
	    }
    } else {
	    d_writer.writeCommentLine("user defined static methods: (none)");
    }

    d_writer.println();
    List nonstatic_methods = (List) d_ext.getNonstaticMethods(false);
    if (nonstatic_methods.size() > 0) {
	    d_writer.writeCommentLine("user defined non-static methods:");
	    for (Iterator m = nonstatic_methods.iterator(); m.hasNext(); ) {
        Method method = (Method) m.next();
        if (!method.isAbstract()) {
          System.out.println("Generating method: method.getShortMethodName()");
          generateMethod(method);
        }
	    }
    } else {
	    d_writer.writeCommentLine("user defined non-static methods: (none)");
    }
    d_writer.println();
  }

  private void generateMethod( Method method ) 
  {
    if ( method == null ) { return; }
	
    d_writer.writeComment (method, true);

    d_writer.print("public ");
    d_writer.println(method.getReturnType().getTypeString());  // BRN todo: may need to convert string
    d_writer.print(method.getShortMethodName());
	
    if (method.getArgumentList().size() > 0) {
	    d_writer.println(" (");
	    d_writer.increaseTabLevel();
	    generateArgumentList(method);
	    d_writer.println(" ) ");
	    d_writer.decreaseTabLevel();
	    generateThrowsList(method, false); 
	    d_writer.println("{");
    } else {
	    d_writer.println(" () ");
	    generateThrowsList(method, false); 
	    d_writer.println("{");
    }
    d_writer.increaseTabLevel();

    System.out.println("Inserting return var declaration: " + method.getReturnType().getTypeString() + " _result;");

    Java.declareJavaVariable(d_writer, method.getReturnType(), "_result");
//     d_writer.println(
//                      Java.substitute((String) Java.s_init_java.get(method.getReturnType().getTypeString()),
//                                      method.getReturnType().getTypeString(),
//                                      " _result"));

    d_splicer.splice(d_ext.getSymbolID().getFullName() + "." + 
                     method.getLongMethodName(),
                     d_writer,
                     "insert implementation here");
    
    // Return if not void
    if (method.getReturnType().getType() != Type.VOID) {
      d_writer.println("return _result;");
    }
    d_writer.decreaseTabLevel();
    d_writer.println("}");
    d_writer.println();
  }

  private void generateArgumentList( Method method )  // BRN todo
  {
    if ( method == null ) { return; }
	
    // Put argument list generation here
	
  }

  private void generateThrowsList( Method method, boolean s )  // BRN todo
  {
    if ( method == null ) { return; }
	
    // Put throws list generation here
	
  }
}
