// 
// File:        arraytests.cc
// Copyright:   (c) 2001 The Regents of the University of California
// Release:     $Name: release-0-8-4 $
// Revision:    @(#) $Revision: 1.14 $
// Date:        $Date: 2003/04/04 18:38:15 $
// Description: Simple test on the ArrayTest static methods
// 
// 

#include <iostream>
using namespace std;

#include "ArrayTest_ArrayOps.hh"
using ArrayTest::ArrayOps;

const char * result_string[] = { "FAIL", "XFAIL", "PASS", "XPASS", "UNSUPPORTED"  };
const int unsupported = 4;
const int xpass = 3;
const int pass = 2;
const int xfail = 1;
const int fail = 0;

static const int TEST_SIZE = 345; /* size of one dimensional arrays */
static const int TEST_DIM1 = 17; /* first dimension of 2-d arrays */
static const int TEST_DIM2 = 13; /* second dimension of 2-d arrays */

void declare_part( int *part_no );
void end_part( int part_no, int result, int *final_result );


#define MYASSERT( AAA ) \
  declare_part( &part_no ); \
  cout << "COMMENT: " <<  #AAA << endl; \
  if ( AAA ) result = pass; \
  else result = fail;  \
  end_part( part_no, result, &final_result );

#define MYXFAIL( AAA ) \
  declare_part( &part_no ); \
  cout << "COMMENT: " << #AAA << endl; \
  if ( AAA ) result = xpass; \
  else result = xfail; \
  end_part( part_no, result, &final_result );

#define MYBROKEN( AAA ) \
  declare_part( &part_no ); \
  cout << "COMMENT: " << #AAA << endl; \
  end_part( part_no, unsupported, &final_result );


static double powTwo(register int32_t i)
{
  register double result = 1;
  if (i >= 0) {
    while (i--) {
      result *= 2;
    }
  }
  else {
    while (i++) {
      result *= 0.5;
    }
  }
  return result;
}

static float fpowTwo(register int32_t i)
{
  register float result = 1;
  static volatile float forcestore;
  if (i >= 0) {
    while (i--) {
      result *= 2.0F;
    }
  }
  else {
    while (i++) {
      result *= 0.5F;
    }
  }
  forcestore = result;
  return forcestore;
}

static SIDL::array<double>
makeDTestMatrix(int32_t rows, int32_t cols) {
  int32_t lower[2] = {0, 0};
  int32_t upper[2] = {rows-1, cols-1};
  SIDL::array<double> result = SIDL::array<double>::createRow(2,lower,upper);
  for( int32_t i=0; i<rows; ++i ) {
    for( int32_t j=0;  j<cols; ++j ) {
      switch(rand() % 3) {
      case 0:
        SIDLArrayElem2(result._get_ior(),i,j) = powTwo(i-j);
        break;
      case 1:
        SIDL_double__array_set2(result._get_ior(), i, j, powTwo(i-j));
        break;
      case 2:
        result.set(i, j, powTwo(i-j));
        break;
      }
    }
  }
  return result;
}

static SIDL::array<float>
makeFTestMatrix(int32_t rows, int32_t cols) {
  int32_t lower[2] = {0, 0};
  int32_t upper[2] = { rows-1, cols-1};
  SIDL::array<float> result = SIDL::array<float>::createRow(2, lower, upper);
  for( int32_t i=0; i<rows; ++i) {
    for( int32_t j=0; j<cols; ++j) {
      switch(rand() % 3) {
      case 0:
        SIDLArrayElem2(result._get_ior(),i,j) = powTwo(i-j);
        break;
      case 1:
        SIDL_float__array_set2(result._get_ior(), i, j, powTwo(i-j));
        break;
      case 2:
        result.set(i, j, powTwo(i-j));
        break;
      }
    }
  }
  return result;
}

static SIDL::array<SIDL::dcomplex> 
makeDCTestMatrix(int32_t rows, int32_t cols) {
  int32_t lower[2] = {0, 0};
  int32_t upper[2] = {rows-1, cols-1};
  SIDL::array<SIDL::dcomplex> result = SIDL::array<SIDL::dcomplex>::createRow(2, lower, upper);
  for(int32_t i=0; i<rows; ++i) {
    for(int32_t j=0; j<cols; ++j) {
      complex<double> tmp( powTwo(i), powTwo(-j) );
      struct SIDL_dcomplex tmp2;
      tmp2.real = tmp.real();
      tmp2.imaginary = tmp.imag();
      switch(rand() % 3) {
      case 0:
        SIDLArrayElem2(result._get_ior(),i,j) = tmp2;
        break;
      case 1:
        SIDL_dcomplex__array_set2(result._get_ior(), i, j, tmp2);
        break;
      case 2:
	result.set(i, j, tmp);
        break;
      }
    }
  }
  return result;
}

static SIDL::array<SIDL::fcomplex>
makeFCTestMatrix(int32_t rows, int32_t cols) {
  int32_t lower[2] = {0,0};
  int32_t upper[2] = { rows-1, cols-1};
  SIDL::array<SIDL::fcomplex> result = SIDL::array<SIDL::fcomplex>::createRow(2, lower, upper);
  for(int32_t i=0; i<rows; ++i) {
    for(int32_t j=0; j<cols; ++j) {
      complex<float> tmp2( fpowTwo(i), fpowTwo(-j) );
      struct SIDL_fcomplex tmp;
      tmp.real = fpowTwo(i);
      tmp.imaginary = fpowTwo(-j);
      switch(rand() % 3) {
      case 0:
        SIDLArrayElem2(result._get_ior(),i,j) = tmp;
        break;
      case 1:
        SIDL_fcomplex__array_set2(result._get_ior(), i, j, tmp);
        break;
      case 2:
        result.set(i, j, tmp2);
        break;
      }
    }
  }
  return result;
}

static int32_t intFunc(const int dimen, const int32_t ind[])
{
  int32_t result = 1;
  int i;
  for(i = 0;i < dimen; ++i){
    result *= (ind[i] + (i + 1));
  }
  return result;
}

static bool
next(const int dimen, int32_t ind[],
     const int32_t lower[], const int32_t upper[])
{
  int i = 0;
  while ((i < dimen) && (++(ind[i]) > upper[i])) {
    ind[i] = lower[i];
    ++i;
  }
  return i < dimen;
}

static SIDL::array<int32_t>
makeIntTestMatrix(int dimen)
{
  struct SIDL::array<int32_t> result;
  static const int32_t lower[] = {0, 0, 0, 0};
  static const int32_t upper[] = {3, 3, 2, 2};
  int32_t ind[] = {0, 0, 0, 0};
  int32_t value;
  result  = SIDL::array<int32_t>::createCol(dimen, lower, upper);
  do {
    value = intFunc(dimen, ind);
    switch(rand() % 2) {
    case 0:
      if (dimen == 3) {
	result.set(ind[0], ind[1], ind[2], value);
      }
      else {
        result.set(ind[0], ind[1], ind[2], ind[3], value);
      }
      break;
    case 1:
      result.set(ind, value);
      break;
    }
  } while (next(dimen, ind, lower, upper));
  return result;
}

/**
 * Fill the stack with random junk.
 */
int clearstack(int magicNumber) {
  int chunk[2048], i;
  for(i = 0; i < 2048; i++){
    chunk[i] = rand() + magicNumber;
  }
  for(i = 0; i < 16; i++){
    magicNumber += chunk[rand() & 2047];
  }
  return magicNumber;
}


int main(int argc, char **argv) {
  int result = pass;
  int final_result = pass;
  int part_no = 0;
  int magicNumber = 1;

  cout << "NPARTS -1\n";

  {
    magicNumber = clearstack(magicNumber);
    SIDL::array<bool> barray = ArrayOps::createBool(TEST_SIZE);
    MYASSERT(barray._not_nil());
    MYASSERT(ArrayOps::checkBool(barray) == true);
    MYASSERT(ArrayOps::reverseBool(barray, true) == true);
    barray.deleteRef();
    magicNumber = clearstack(magicNumber);
  }

  {
    SIDL::array<bool> barray;
    ArrayOps::makeBool(218, barray);
    MYASSERT(ArrayOps::checkBool(barray) == true);
    MYASSERT(ArrayOps::reverseBool(barray, false) == true);
    MYASSERT(ArrayOps::checkBool(barray) == false);
    barray.deleteRef();
    magicNumber = clearstack(magicNumber);
  }
  
  {
    SIDL::array<bool> barray;
    ArrayOps::makeBool(9, barray);
    MYASSERT(ArrayOps::reverseBool(barray, false) == true);
    MYASSERT(ArrayOps::checkBool(barray) == true);
    barray.deleteRef();
    magicNumber = clearstack(magicNumber);
  }
  
  {
    magicNumber = clearstack(magicNumber);
    SIDL::array<char> carray = ArrayOps::createChar(TEST_SIZE);
    MYASSERT(carray._not_nil());
    MYASSERT(ArrayOps::checkChar(carray) == true);
    MYASSERT(ArrayOps::reverseChar(carray, true) == true);
    carray.deleteRef();
    magicNumber = clearstack(magicNumber);
  }

  {
    SIDL::array<char> carray;
    ArrayOps::makeChar(218, carray);
    MYASSERT(ArrayOps::checkChar(carray) == true);
    MYASSERT(ArrayOps::reverseChar(carray, false) == true);
    MYASSERT(ArrayOps::checkChar(carray) == false);
    carray.deleteRef();
    magicNumber = clearstack(magicNumber);
  }
  
  {
    magicNumber = clearstack(magicNumber);
    SIDL::array<int32_t> iarray = ArrayOps::createInt(TEST_SIZE);
    MYASSERT(iarray._not_nil());
    MYASSERT(ArrayOps::checkInt(iarray) == true);
    MYASSERT(ArrayOps::reverseInt(iarray, true) == true);
    iarray.deleteRef();
    magicNumber = clearstack(magicNumber);
  } 

  {
    SIDL::array<int32_t> iarray;
    ArrayOps::makeInt(218, iarray);
    MYASSERT(ArrayOps::checkInt(iarray) == true);
    MYASSERT(ArrayOps::reverseInt(iarray, false) == true);
    MYASSERT(ArrayOps::checkInt(iarray) == false);
    iarray.deleteRef();
    magicNumber = clearstack(magicNumber);
  } 

  {
    magicNumber = clearstack(magicNumber);
    SIDL::array<int64_t> larray = ArrayOps::createLong(TEST_SIZE);
    MYASSERT(larray._not_nil());
    MYASSERT(ArrayOps::checkLong(larray) == true);
    MYASSERT(ArrayOps::reverseLong(larray, true) == true);
    larray.deleteRef();
    magicNumber = clearstack(magicNumber);
  } 

  { 
    SIDL::array<int64_t> larray;
    ArrayOps::makeLong(218, larray);
    MYASSERT(ArrayOps::checkLong(larray) == true);
    MYASSERT(ArrayOps::reverseLong(larray, false) == true);
    MYASSERT(ArrayOps::checkLong(larray) == false);
    larray.deleteRef();
    magicNumber = clearstack(magicNumber);
  } 

  {
    magicNumber = clearstack(magicNumber);
    SIDL::array<string> sarray = ArrayOps::createString(TEST_SIZE);
    MYASSERT(sarray._not_nil());
    MYASSERT(ArrayOps::checkString(sarray) == true);
    MYASSERT(ArrayOps::reverseString(sarray, true) == true);
    sarray.deleteRef();
    magicNumber = clearstack(magicNumber);
  }

  { 
    SIDL::array<string> sarray; 
    ArrayOps::makeString(218, sarray);
    MYASSERT(ArrayOps::checkString(sarray) == true);
    MYASSERT(ArrayOps::reverseString(sarray, false) == true);
    MYASSERT(ArrayOps::checkString(sarray) == false);
    sarray.deleteRef();
    magicNumber = clearstack(magicNumber);
  }

  {
    magicNumber = clearstack(magicNumber);
    SIDL::array<double> darray = ArrayOps::createDouble(TEST_SIZE);
    MYASSERT(darray._not_nil());
    MYASSERT(ArrayOps::checkDouble(darray) == true);
    MYASSERT(ArrayOps::reverseDouble(darray, true) == true);
    darray.deleteRef();
    magicNumber = clearstack(magicNumber);
  } 

  {
    SIDL::array<double> darray;
    ArrayOps::makeDouble(218, darray);
    MYASSERT(ArrayOps::checkDouble(darray) == true);
    MYASSERT(ArrayOps::reverseDouble(darray, false) == true);
    MYASSERT(ArrayOps::checkDouble(darray) == false);
    darray.deleteRef();
    magicNumber = clearstack(magicNumber);
  } 
  
  { 
    magicNumber = clearstack(magicNumber);
    SIDL::array<float> farray = ArrayOps::createFloat(TEST_SIZE);
    MYASSERT(farray._not_nil());
    MYASSERT(ArrayOps::checkFloat(farray) == true);
    MYASSERT(ArrayOps::reverseFloat(farray, true) == true);
    farray.deleteRef();
    magicNumber = clearstack(magicNumber);
  }

  { 
    SIDL::array<float> farray; 
    ArrayOps::makeFloat(218, farray);
    MYASSERT(ArrayOps::checkFloat(farray) == true);
    MYASSERT(ArrayOps::reverseFloat(farray, false) == true);
    MYASSERT(ArrayOps::checkFloat(farray) == false);
    farray.deleteRef();
    magicNumber = clearstack(magicNumber);
  } 
  
  {
    magicNumber = clearstack(magicNumber);
    SIDL::array<SIDL::fcomplex> fcarray = ArrayOps::createFcomplex(TEST_SIZE);
    MYASSERT(fcarray._not_nil());
    MYASSERT(ArrayOps::checkFcomplex(fcarray) == true);
    MYASSERT(ArrayOps::reverseFcomplex(fcarray, true) == true);
    fcarray.deleteRef();
    magicNumber = clearstack(magicNumber);
  } 

  { 
    SIDL::array<SIDL::fcomplex> fcarray;
    ArrayOps::makeFcomplex(218, fcarray);
    MYASSERT(ArrayOps::checkFcomplex(fcarray) == true);
    MYASSERT(ArrayOps::reverseFcomplex(fcarray, false) == true);
    MYASSERT(ArrayOps::checkFcomplex(fcarray) == false);
    fcarray.deleteRef();
    magicNumber = clearstack(magicNumber);
  } 
  
  {
    magicNumber = clearstack(magicNumber);
    SIDL::array<SIDL::dcomplex> dcarray = ArrayOps::createDcomplex(TEST_SIZE);
    MYASSERT(dcarray._not_nil());
    MYASSERT(ArrayOps::checkDcomplex(dcarray) == true);
    MYASSERT(ArrayOps::reverseDcomplex(dcarray, true) == true);
    dcarray.deleteRef();
    magicNumber = clearstack(magicNumber);
  } 

  { 
    SIDL::array<SIDL::dcomplex> dcarray;
    ArrayOps::makeDcomplex(218, dcarray);
    MYASSERT(ArrayOps::checkDcomplex(dcarray) == true);
    MYASSERT(ArrayOps::reverseDcomplex(dcarray, false) == true);
    MYASSERT(ArrayOps::checkDcomplex(dcarray) == false);
    dcarray.deleteRef();
    magicNumber = clearstack(magicNumber);
  } 

  { 
    magicNumber = clearstack(magicNumber);
    SIDL::array<double> darray = makeDTestMatrix(TEST_DIM1,TEST_DIM2);
    MYASSERT(ArrayOps::check2Double(darray) == true);
    darray.deleteRef();
  } 

  { 
    magicNumber = clearstack(magicNumber);
    SIDL::array<float> farray = makeFTestMatrix(TEST_DIM1,TEST_DIM2);
    MYASSERT(ArrayOps::check2Float(farray) == true);
    farray.deleteRef();
  } 

  { 
    magicNumber = clearstack(magicNumber);
    SIDL::array<SIDL::dcomplex> dcarray = makeDCTestMatrix(TEST_DIM1,TEST_DIM2);
    MYASSERT(ArrayOps::check2Dcomplex(dcarray) == true);
    dcarray.deleteRef();
  } 

  { 
    magicNumber = clearstack(magicNumber);
    SIDL::array<SIDL::fcomplex> fcarray = makeFCTestMatrix(TEST_DIM1,TEST_DIM2);
    MYASSERT(ArrayOps::check2Fcomplex(fcarray) == true);
    fcarray.deleteRef();
  } 

  {
    magicNumber = clearstack(magicNumber);
    SIDL::array<int32_t> iarray = makeIntTestMatrix(3);
    MYASSERT(ArrayOps::check3Int(iarray) == true);
    iarray.deleteRef();
  }

  {
    magicNumber = clearstack(magicNumber);
    SIDL::array<int32_t> iarray = makeIntTestMatrix(4);
    MYASSERT(ArrayOps::check4Int(iarray) == true);
    iarray.deleteRef();
  }

  cout << "TEST_RESULT " << result_string[ final_result ] << endl;
  return 0;
}

void declare_part( int * part_no ) {
  ++(*part_no);
  cout << "PART " << *part_no << endl;
}

void end_part( int part_no, int result, int *final_result ) {
  cout << "RESULT " << part_no << " " << result_string[ result ] << endl;
  if ( result == xpass ) {
    *final_result = fail;
  } else if ( result == unsupported && *final_result == pass ) { 
    *final_result = xfail;
  } else if ( result < *final_result ) { 
    *final_result = result;
  }
}

